#
# Copyright (c) 2013, Sascha Peilicke <saschpe@gmx.de>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program (see the file COPYING); if not, write to the
# Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
"""
:mod:`rapport` -- Work Report Generator
=======================================

.. automodule:: rapport
   :synopsis: Work Report Generator
.. moduleauthor:: Sascha Peilicke <saschpe@gmx.de>
"""

__author__ = "Sascha Peilicke <saschpe@gmx.de>"
__version_info__ = ("0", "0", "8")
__version__ = ".".join(__version_info__)
